﻿USE AdventureWorks2008R2;
GO
-- [D1]
SELECT	p.BusinessEntityID,
		p.FirstName,
		p.MiddleName,
		p.LastName
FROM	Person.Person p -- IX_Person_LastName_FirstName_MiddleName
 WHERE	p.LastName=N'Norman' AND p.FirstName=N'Laura' AND p.MiddleName<N'H'			-- SARG: MiddleName incepe cu A,B,... sau G
-- WHERE	p.LastName=N'Norman' AND p.FirstName=N'Laura' AND (p.MiddleName>=N'A' AND p.MiddleName<N'H') -- SARG: MiddleName incepe cu A,B,... sau G
-- WHERE	p.LastName=N'Norman' AND p.FirstName=N'Laura' AND p.MiddleName IS NULL		-- SARG
-- WHERE	p.LastName=N'Norman' AND p.FirstName=N'Laura' AND p.MiddleName IS NOT NULL	-- SARG
-- WHERE	p.LastName=N'Norman' AND p.FirstName<N'D' AND p.MiddleName=N'F'				-- Non SARG
ORDER BY p.LastName, p.FirstName, p.MiddleName

-- [D2] SkipScan (fara partionare)
-- Vezi: http://sqlblog.com/blogs/joe_chang/archive/2011/06/13/oracle-index-skip-scan.aspx
-- SSMS > Ctrl+M
SET STATISTICS IO ON;
GO
SELECT	COUNT(*)
FROM	Sales.SalesOrderHeader h
WHERE	h.OrderDate BETWEEN '2008-07-01T00:00:00.000' AND '2008-07-15T23:59:59.997';
GO
SET STATISTICS IO OFF;
GO
-- Table 'SalesOrderHeader'. Scan count 1, logical reads 5

-- Dezactivez pentru această demonstrație indexul care are cheia (OrderDate)
-- Indexul (ShipMethodID,OrderDate) este [în continuare] activ (IX_SalesOrderHeader_ShipMethodID_OrderDate_#_ShipDate)
ALTER INDEX IX_SalesOrderHeader_OrderDate_#_ShipDate_SubTotal_CustomerID
ON Sales.SalesOrderHeader
DISABLE;
GO

SET STATISTICS IO ON;
GO
SELECT	COUNT(*) AS OrderCount
FROM	Sales.SalesOrderHeader h -- WITH(INDEX=IX_SalesOrderHeader_ShipMethodID_OrderDate_#_ShipDate)
WHERE	h.OrderDate BETWEEN '2008-07-01T00:00:00.000' AND '2008-07-15T23:59:59.997';
GO
SET STATISTICS IO OFF;
GO
-- Table 'SalesOrderHeader'. Scan count 1, logical reads 119

SET STATISTICS IO ON;
GO
SELECT	COUNT(*) AS OrderCount
FROM	Purchasing.ShipMethod sm -- "FK elimination"
JOIN	Sales.SalesOrderHeader h ON sm.ShipMethodID=h.ShipMethodID
WHERE	h.OrderDate BETWEEN '2008-07-01T00:00:00.000' AND '2008-07-15T23:59:59.997';
GO
SET STATISTICS IO OFF;
GO
-- Table 'SalesOrderHeader'. Scan count 1, logical reads 76

SET STATISTICS IO ON;
GO
SELECT	MIN(sm.ShipMethodID) AS DummyCol, -- Pentru a bloca "FK elimination"	
		COUNT(*) AS OrderCount
FROM	Purchasing.ShipMethod sm 
JOIN	Sales.SalesOrderHeader h ON sm.ShipMethodID=h.ShipMethodID
WHERE	h.OrderDate BETWEEN '2008-07-01T00:00:00.000' AND '2008-07-15T23:59:59.997';
GO
SET STATISTICS IO OFF;
GO
/*
Table 'SalesOrderHeader'. Scan count 5, logical reads 11
Table 'ShipMethod'. Scan count 1, logical reads 2
*/

ALTER INDEX IX_SalesOrderHeader_OrderDate_#_ShipDate_SubTotal_CustomerID
ON Sales.SalesOrderHeader
REBUILD;
GO

-- [D3]
CREATE SCHEMA Demo;
GO
SELECT	h.*
INTO	Demo.SalesOrderHeader
FROM	Sales.SalesOrderHeader h;
GO
CREATE UNIQUE CLUSTERED INDEX PK_SalesOrderHeader_SalesOrderID
ON Demo.SalesOrderHeader(SalesOrderID);
GO
CREATE UNIQUE INDEX IUF_SalesOrderHeader_OrderDate_CustomerID_SubTotalLsThn10
ON Demo.SalesOrderHeader (OrderDate,CustomerID)
-- INCLUDE(SubTotal)
WHERE SubTotal<10
GO
CREATE UNIQUE INDEX IUF_SalesOrderHeader_OrderDate_CustomerID_FreightGrThn5000
ON Demo.SalesOrderHeader (OrderDate,CustomerID)
-- INCLUDE (Freight)
WHERE Freight>5000
GO
/*
DROP INDEX IUF_SalesOrderHeader_OrderDate_CustomerID_SubTotalLsThn10
ON Demo.SalesOrderHeader

DROP INDEX IUF_SalesOrderHeader_OrderDate_CustomerID_FreightGrThn5000
ON Demo.SalesOrderHeader
*/

SELECT	h.OrderDate, h.CustomerID, h.SalesOrderID
FROM	Demo.SalesOrderHeader h
WHERE	SubTotal<10

-- SARG: Index Seek
SELECT	h.OrderDate, h.CustomerID, h.SalesOrderID
FROM	Demo.SalesOrderHeader h
WHERE	SubTotal<10
AND		OrderDate>='20061001' AND OrderDate<='20071001'

-- Index Scan (fara INCLUDED)
SELECT	h.OrderDate, h.CustomerID, h.SalesOrderID
FROM	Demo.SalesOrderHeader h
WHERE	SubTotal<5
AND		OrderDate>='20061001' AND OrderDate<='20071001'

-- SARG: Index Seek (logic UNION)
SELECT	h.OrderDate, h.CustomerID, h.SalesOrderID
FROM	Demo.SalesOrderHeader h
WHERE	(SubTotal<10
OR		Freight>5000) AND OrderDate>='20061001' AND OrderDate<='20071001'
