﻿USE AdventureWorks2008R2;
DBCC FREEPROCCACHE; -- Nu executati aceasta comanda pe un server folosit in productie!
GO
-- [D1]: Din ce este format ?
	-- SQL Server Management Studio|SSMS: Ctrl+M > F5(Execute)
	-- [D1.1] Inner Join > Nested Loops
	DECLARE @TerritoryID INT=10;
	SELECT	soh.SalesOrderID
	FROM	Sales.SalesPerson sp INNER JOIN Sales.SalesOrderHeader soh 
	ON		soh.SalesPersonID=sp.BusinessEntityID
	WHERE	sp.TerritoryID=@TerritoryID -- United Kingdom
	GO 2
	-- [D1.2] Inner join > Merge Join, Warning: The join order has been enforced because a local join hint is used.
	DECLARE @TerritoryID INT=10;
	SELECT	soh.SalesOrderID
	FROM	Sales.SalesPerson sp INNER MERGE JOIN Sales.SalesOrderHeader soh 
	ON		soh.SalesPersonID=sp.BusinessEntityID -- In acest caz, MERGE nu este optim
	WHERE	sp.TerritoryID=@TerritoryID -- United Kingdom
	GO 2
	GO

-- [D2]: Interceptarea planurilor de executie
	CREATE PROCEDURE Sales.uspGetOrdersForTerritory(@TerritoryID INT)
	AS
	BEGIN
		SELECT	soh.SalesOrderID
		FROM	Sales.SalesPerson sp INNER JOIN Sales.SalesOrderHeader soh 
		ON		soh.SalesPersonID=sp.BusinessEntityID
		WHERE	sp.TerritoryID=@TerritoryID;
	END;
	GO

	-- [D2.1] 
	-- SSMS: Query > Include actual execution plan (Ctrl+M) > F5 (Execute)
	EXEC Sales.uspGetOrdersForTerritory @TerritoryID=1;
	GO
	-- SSMS: Query > Display estimated execution plan (Ctrl+L)

	-- [D2.2] Query cache: planuri estimate
	SELECT	src.dbid,src.text,
			pl.dbid,pl.query_plan, -- Continutul XML poate fi salvat intr-un fisier *.sqlplan (se deschide cu SSMS)
			cache.* 
	FROM	sys.dm_exec_cached_plans cache
	CROSS APPLY sys.dm_exec_sql_text(cache.plan_handle) src
	CROSS APPLY sys.dm_exec_query_plan(cache.plan_handle) pl
	WHERE	cache.cacheobjtype LIKE N'Compiled Plan%'
	--AND		src.objectid=OBJECT_ID(N'Sales.uspGetOrdersForTerritory') AND src.dbid=DB_ID() -- Pentru proceduri stocate
	--AND	src.text LIKE '%D1%' -- Pentru cereri adhoc

-- [D3]: Intepretarea planurilor de execuție
	/*
	CREATE NONCLUSTERED INDEX [IX_SalesOrderHeader_ShipMethodID_OrderDate_#_ShipDate] 
	ON Sales.SalesOrderHeader 
	(
		ShipMethodID ASC,
		OrderDate ASC
	)
	INCLUDE (ShipDate)
	*/
	-- Ultimele 10 comenzi din 2008 cu modalitatea de livrare N'CARGO TRANSPORT 5' 
	-- ordonate dupa data comenzii și data livrării DESC
	-- Ctrl+M + F5
	SELECT	x.SalesOrderID,x.OrderDate,x.ShipDate,
			DATEDIFF(DAY,x.OrderDate,x.ShipDate) AS ShipDays -- Expresia a fost scrisă aici pentru a demonstra faptul că ordinea de execuție fizică poate să difere de ce logică (fizic: expresie este evaluată înainte de TOP vs. logic: expresia este evaluată după TOP)
	FROM	Purchasing.ShipMethod sm
	CROSS APPLY(
		SELECT	TOP(10)
				soh.SalesOrderID,soh.OrderDate,soh.ShipDate
		FROM	Sales.SalesOrderHeader soh
		WHERE	soh.ShipMethodID=sm.ShipMethodID
		AND		soh.OrderDate>='20070101'
		AND		soh.OrderDate<'20080101'
		ORDER BY soh.OrderDate DESC
	)x
	WHERE	sm.Name=N'CARGO TRANSPORT 5'
	ORDER BY x.OrderDate DESC, x.ShipDate DESC
	GO

-- [D4] {[Clustered] Index|Table} Scan
	-- Index existent
	/*
	CREATE NONCLUSTERED INDEX IX_SalesOrderHeader_OrderDate 
	ON Sales.SalesOrderHeader(OrderDate ASC);
	*/

	-- {Exemplu negativ}
		SET STATISTICS IO ON;
		GO
		DECLARE @StartDate DATETIME,@EndDate DATETIME;
		SET		@StartDate='20080701'
		SET		@EndDate='20080715'
		SELECT	soh.SalesOrderID,
				soh.OrderDate,
				soh.ShipDate
		FROM	Sales.SalesOrderHeader soh -- WITH(FORCESEEK) -- Doar pentru Test 2
		WHERE	soh.OrderDate>=@StartDate
		AND		soh.OrderDate<@EndDate;
		GO
		SET STATISTICS IO OFF;
		GO
		-- Test 1 (IX_SalesOrderHeader_ShipMethodID_OrderDate_#_ShipDate): 
		-- Table 'SalesOrderHeader'. Scan count 1, logical reads 119
		/*
		Vezi: Missing Index Details ...
		CREATE NONCLUSTERED INDEX [<Name of Missing Index, sysname,>]
		ON [Sales].[SalesOrderHeader] ([OrderDate])
		INCLUDE ([SalesOrderID],[ShipDate])
		*/

		-- Test 2 (FORCESEEK > IX_SalesOrderHeader_OrderDate)
		-- Table 'SalesOrderHeader'. Scan count 1, logical reads 1351
		/*
		Vezi: Missing Index Details ...
		*/

		-- Test 3 (fara FORCESEEK)
		/*
		-- DROP INDEX [IX_SalesOrderHeader_OrderDate_#_ShipDate] ON [Sales].[SalesOrderHeader]
		CREATE NONCLUSTERED INDEX IX_SalesOrderHeader_OrderDate
		ON [Sales].[SalesOrderHeader] ([OrderDate])
		INCLUDE ([ShipDate]) -- [SalesOrderID] este cheia indexului [UNIQUE] clustered
		WITH(DROP_EXISTING=ON);
		GO
		EXEC sp_rename 
			@objname=N'Sales.SalesOrderHeader.IX_SalesOrderHeader_OrderDate',
			@newname=N'IX_SalesOrderHeader_OrderDate_#_ShipDate',
			@objtype=N'INDEX';
		GO
		*/
		-- Test 3 fără WITH(FORCESEEK)
		-- Table 'SalesOrderHeader'. Scan count 1, logical reads 4
		-- plus
		GO
		CREATE PROCEDURE Sales.uspGetOrders(@StartDate DATETIME,@EndDate DATETIME)
		AS
		BEGIN
			SELECT	soh.SalesOrderID,
					soh.OrderDate,
					soh.ShipDate
			FROM	Sales.SalesOrderHeader soh
			WHERE	soh.OrderDate>=@StartDate
			AND		soh.OrderDate<@EndDate;
		END
		GO
		EXEC Sales.uspGetOrders @StartDate='20080701',@EndDate='20080715';
		GO

	-- {Exemplu pozitiv/acceptabil}
		SET STATISTICS IO ON;
		GO
		SELECT	sm.ShipMethodID,sm.ShipRate, soh.SalesOrderID
		FROM	Purchasing.ShipMethod sm 
		JOIN	Sales.SalesOrderHeader soh ON soh.ShipMethodID=sm.ShipMethodID
		WHERE	sm.ShipRate<1;
		GO
		SET STATISTICS IO OFF;
		GO
		-- Table 'ShipMethod'. Scan count 1, logical reads 2

-- [D5] {Key|RID} Lookup
	-- {Exemplu negativ}
		ALTER PROCEDURE Sales.uspGetOrders(@StartDate DATETIME,@EndDate DATETIME)
		AS
		BEGIN
			SELECT	soh.SalesOrderID,
					soh.OrderDate,
					soh.ShipDate,
					soh.SubTotal
			FROM	Sales.SalesOrderHeader soh
			WHERE	soh.OrderDate>=@StartDate
			AND		soh.OrderDate<@EndDate;
		END
		GO

		-- Test 1 (Parameter Sniffing)
		SET STATISTICS IO ON;
		EXEC Sales.uspGetOrders @StartDate='20080715',@EndDate='20080717';
		SET STATISTICS IO OFF;
		GO 
		-- Table 'SalesOrderHeader'. Scan count 1, logical reads 213

		-- Test 2
		SET STATISTICS IO ON;
		EXEC Sales.uspGetOrders @StartDate='20040101',@EndDate='20080717';
		SET STATISTICS IO OFF;
		GO
		-- Table 'SalesOrderHeader'. Scan count 1, logical reads 95024

		-- Test 3
		/*
		-- DROP INDEX [IX_SalesOrderHeader_OrderDate_#_ShipDate] ON [Sales].[SalesOrderHeader]
		CREATE NONCLUSTERED INDEX IX_SalesOrderHeader_OrderDate_#_ShipDate
		ON [Sales].[SalesOrderHeader] ([OrderDate])
		INCLUDE (ShipDate,SubTotal)
		WITH(DROP_EXISTING=ON);
		GO
		EXEC sp_rename 
			@objname=N'Sales.SalesOrderHeader.IX_SalesOrderHeader_OrderDate_#_ShipDate',
			@newname=N'IX_SalesOrderHeader_OrderDate_#_ShipDate_SubTotal',
			@objtype=N'INDEX';
		GO
		*/
		-- plus se repeta Test 2
		SET STATISTICS IO ON;
		EXEC Sales.uspGetOrders @StartDate='20040101',@EndDate='20080717';
		SET STATISTICS IO OFF;
		GO
		-- Table 'SalesOrderHeader'. Scan count 1, logical reads 133

	-- {Exemplu pozitiv/acceptabil}
		SET STATISTICS IO ON;
		DECLARE @Date DATETIME;
		SET		@Date='20080701';
		SELECT	soh.SalesOrderID,
				soh.OrderDate,
				soh.ShipDate,
				soh.SubTotal,
				soh.TaxAmt,
				soh.Freight
		FROM	Sales.SalesOrderHeader soh
		WHERE	soh.OrderDate=@Date; -- Predicatul are selectivitate mare și relativ constantă
		SET STATISTICS IO OFF;
		GO

-- [D6] Sort
	-- Cauzat de ORDER BY, MERGE JOIN, DISTINCT
	-- {Exemplu negativ}
		ALTER PROCEDURE Sales.uspGetOrders(@StartDate DATETIME,@EndDate DATETIME)
		AS
		BEGIN
			SELECT	soh.SalesOrderID,
					soh.OrderDate,
					soh.ShipDate,
					soh.SubTotal,
					soh.CustomerID,
					oa.PreviousOrderDate,
					DATEDIFF(DAY,oa.PreviousOrderDate,soh.OrderDate) AS DaysFromPreviousOrder
			FROM	Sales.SalesOrderHeader soh
			OUTER APPLY(
				SELECT	TOP(1)
						a.OrderDate AS PreviousOrderDate
				FROM	Sales.SalesOrderHeader a
				WHERE	a.CustomerID=soh.CustomerID
				AND		a.OrderDate<=soh.OrderDate
				AND		a.SalesOrderID<>soh.SalesOrderID -- a.SalesOrderID<soh.SalesOrderID
				ORDER BY a.OrderDate DESC
			) oa
			WHERE	soh.OrderDate>=@StartDate
			AND		soh.OrderDate<@EndDate;
		END
		GO
		-- Test 1
		SET STATISTICS IO ON;
		EXEC Sales.uspGetOrders @StartDate='20080715',@EndDate='20080717';
		SET STATISTICS IO OFF;
		GO
		
		-- Test 2
		/*
		CREATE NONCLUSTERED INDEX IX_SalesOrderHeader_OrderDate_#_ShipDate_SubTotal
		ON [Sales].[SalesOrderHeader] ([OrderDate])
		INCLUDE (ShipDate,SubTotal,CustomerID)
		WITH(DROP_EXISTING=ON);
		GO
		EXEC sp_rename 
			@objname=N'Sales.SalesOrderHeader.IX_SalesOrderHeader_OrderDate_#_ShipDate_SubTotal',
			@newname=N'IX_SalesOrderHeader_OrderDate_#_ShipDate_SubTotal_CustomerID',
			@objtype=N'INDEX';
		GO
		*/

		/*
		CREATE NONCLUSTERED INDEX IX_SalesOrderHeader_CustomerID
		ON [Sales].[SalesOrderHeader] (CustomerID,OrderDate)
		WITH(DROP_EXISTING=ON);
		GO
		EXEC sp_rename 
			@objname=N'Sales.SalesOrderHeader.IX_SalesOrderHeader_CustomerID',
			@newname=N'IX_SalesOrderHeader_CustomerID_OrderDate',
			@objtype=N'INDEX';
		GO
		*/

	-- {Exemplu pozitiv/acceptabil}
	-- Vezi [D3]

-- [D7]
	SELECT	TOP(20)
			src.text AS original_text,
			SUBSTRING(
				src.[text],
				qs.statement_start_offset/2,
				CASE WHEN qs.statement_end_offset=-1 THEN LEN(src.[text])*2 ELSE qs.statement_end_offset/2-qs.statement_start_offset/2 END
			) AS query_text, 
			pl.query_plan,
			qs.execution_count,
			DATEDIFF(DAY,qs.creation_time,qs.last_execution_time) AS days_from_last_compiled,
			qs.creation_time AS last_compile_time,
			qs.total_logical_reads,
			(qs.total_logical_reads*8)/1024. AS [total_logical_reads (MB)],
			qs.total_logical_reads*1./qs.execution_count AS avg_logical_reads,	
			qs.total_elapsed_time*1./qs.execution_count AS avg_elapsed_time,
			qs.*
	FROM	sys.dm_exec_query_stats qs
	CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) src
	CROSS APPLY sys.dm_exec_query_plan(qs.plan_handle) pl
	-- ORDER BY avg_logical_reads DESC;
	-- ORDER BY avg_elapsed_time DESC;

	SELECT	ps.object_id, 
			QUOTENAME(s.name)+'.'+QUOTENAME(o.name) AS usp_name,
			ps.total_logical_reads,
			ps.execution_count,
			p.query_plan,
			ps.*
	FROM	sys.dm_exec_procedure_stats ps -- sau sys.dm_exec_trigger_stats
	JOIN	sys.objects o ON ps.object_id=o.object_id
	JOIN	sys.schemas s ON o.schema_id=s.schema_id
	CROSS APPLY sys.dm_exec_query_plan(ps.plan_handle) p
	WHERE	ps.database_id=DB_ID()
	ORDER BY ps.total_logical_reads*1/ps.execution_count DESC;
	



